import java.io.Serializable;

/**
 * Lab #6
 * CS 2334, Section 0??
 * DATE GOES HERE
 * <P>
 * This class models a contact.  (A person with whom one is connected.)
 * </P>
 * @version 1.0
 */
public class Contact  implements  Comparable<Contact>, Serializable {
	/** Needs a serialVersionUID because the class is Serializable */
	private static final long serialVersionUID = 1L;
	
	/** Last name of the contact */
	private String lastName;

	/** First name of the contact */
	private String firstName;

	/** Email address of the contact */
	private String emailAddress;

	/** Street address of the contact */
	private StreetAddress address;
	
	/**
	 * This is the default (no-arg) constructor for the class.
	 */   
	Contact() {
	}

	/**
	 * This is a constructor used when only name information is available.
	 */   		
	Contact(String lastName, String firstName) {
		this.lastName = lastName;
		this.firstName = firstName;
		this.emailAddress = null;
		address = new StreetAddress(null, null, null, null, null);
	}
	
	/**
	 * This is the constructor used when all contact info is available for this contact.
	 */   
	Contact(String[] tokens) {
		this.lastName = tokens[0].trim();
		this.firstName = tokens[1].trim();
		this.emailAddress = tokens[2].trim();
		this.address = new StreetAddress(tokens[3].trim(), tokens[4].trim(), tokens[5].trim(), tokens[6].trim(), tokens[7].trim());
	} 
	 
	/**
	 * Returns the lastName of the contact.
	 * <P>
	 * @return            The lastName of the contact as a string.
	 */      
	public String getLastName() {
		return lastName;
	}

	/**
	 * Sets the last name of the contact.
	 * <P>
	 * @param             lName     The last name of the contact.
	 */      
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	/**
	 * Returns the firstName of the contact.
	 * <P>
	 * @return            The firstName of the contact as a string.
	 */      
	public String getFirstName() {
		return firstName;
	}


	/**
	 * Sets the first name of the contact.
	 * <P>
	 * @param             fName     The first name of the contact.
	 */      
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}
	
	
	/**
	 * Returns the email address of the contact.
	 * <P>
	 * @return            The email address of the contact as a string.
	 */      
	public String getEmailAddress() {
		return firstName;
	}

	
	/**
	 * Sets the email address of the contact.
	 * <P>
	 * @param             email     The email address of the contact.
	 */      
	public void setEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
	}


	/**
	 * This method returns the attributes of this class as 
	 * a single string.
	 * </P>
	 * @return            String representing the 
	 *                    contents of this object.
	 */
	public String toString() {
		String attributesAsString = lastName + ",  " + firstName;
		if (emailAddress != null) {	
			attributesAsString += ", "+ emailAddress;
		}
		if (address.getState() != null) {
			attributesAsString += ", " + address.toString()+ "\n";
		}
		return (attributesAsString); 	
	}
    
   /**
    * This method compares an instance of Contact with 
    * this instance of Contact to see if they are equal.
    * <P>
    * Algorithm:<br>
    * Compare the last name of the contacts. If it is the same, 
    * then compare the first name of the contacts.
    * </P>
    * @param             otherContact	The other contact with which we are
    * 									comparing this instance of Contact.
    * @return            Returns true if the two instances are 
    *                    equal, false otherwise.
    */   
   public boolean equals(Contact otherContact) {
      // First compare the last names of the contacts.
      if (lastName.equals(otherContact.lastName) 
    		  && firstName.equals(otherContact.firstName)) {
    	  return true;
      }
      return false;
   }
   
   /**
    * This method compares an instance of Contact with 
    * this instance of Contact to determine their ordering.
    * <P>
    * Algorithm:<br>
    * 1. Compare the first names of the contacts.
    * 2. If last names are the same then compare the first names.
    * 3. If first names are the same then compare the email addresses.
    * 
    * </P>
    * @param             otherContact	The other contact with which we are
    * 									comparing this instance of Contact.
    * @return            Returns negative integer if this object is less then otherContact,
    *                    zero if the this object is equal to otherContact,
    *                    positive integer if this object is greater than otherContact.
    */    
   public int compareTo(Contact otherContact) {

      // First compare the first name of the contacts
      if (lastName.compareTo(otherContact.getLastName()) == 0) {
    	  return firstName.compareTo(otherContact.getFirstName());
      }
      else {
    	  return lastName.compareTo(otherContact.getLastName());
      } 
   }
}
