import java.awt.BorderLayout;
import java.awt.event.*;

import javax.swing.*;

/**
 * This class represents the master list view for the MVC lab.
 * It displays the last and first names of all the contacts 
 * in the address book in a JList. The user can select a contact
 * and press the display contact button. The selected
 * contact is then shown in the sub list view.
 *    
 * */
public class MasterListView extends JFrame implements ActionListener {
	/** Needs a serialVersionUID because the class is Serializable */
	private static final long serialVersionUID = 1L;

	//The model for this view.
	private Model model;

	//setup GUI for this view.
	private JButton displayJButton = new JButton("Display Selected Contact Info");
	private DefaultListModel listModel = new DefaultListModel();
	private JList addressBookJList = new JList(listModel);
	private JScrollPane scrollPane = new JScrollPane(addressBookJList); 

	/** The default constructor*/
	MasterListView() {
		setTitle("Last and First Names of Contacts");
		addressBookJList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		add(scrollPane);
		add(displayJButton, BorderLayout.EAST);
		pack();
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLocationRelativeTo(null);
		setVisible(true);
	}

	public void setModel(Model model) {
		/** Set a model */
		this.model = model;
		populateJList();
		if (this.model != null) {
			// Register the view as listener for the model

		}
	}

	/**
	 * This method pulls the last and first name of the contacts 
	 * form the addressBook and stores them in the JList.				
	 */
	private void populateJList() {
		listModel.clear();
		if (model.getAddressBook() != null)
			for (int i = 0; i < model.getAddressBook().size(); i++) {
				listModel.addElement((model.getAddressBook().get(i).getLastName() + ", " +
						model.getAddressBook().get(i).getFirstName()));
			}
	}

	/**
	 * ActionListener for the method.
	 * <P>
	 * 
	 * @param      actionEvent     an event
	 * 					
	 */
	public void actionPerformed(ActionEvent actionEvent) {
		if (actionEvent.getActionCommand().equals("addressBook")) {
			populateJList();
		}
	}

	/**
	 *  @return   JList which contains the contacts    
	 */
	public JList getMasterViewJList() {
		return addressBookJList;
	}

	/**
	 *  @return      the JButton that can be pressed by the user    
	 */
	public JButton getDisplayJButton() {
		return displayJButton;
	}
}