import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;
/**
 * This class represents the Model for the MVC lab.
 * It contains the data that will be shared among various views
 * and the method to manipulate the data. 
 *    
 * */

public class Model {
	//The list of action listeners

	
	//The address book created from the file

	
	//This is the sublist, which will be displayed by SubListView.

	
	/** The default (no-arg) constructor */
	Model() {
		addressBook = null;
		subList = new ArrayList<Contact>();
	}
	
	/**
	 * The constructor for the class.
	 * <P>
	 * 
	 * @param      addressBook 	a list of Contact objects
	 */
	Model(List<Contact> addressBook) {		
		//set the address book
		this.addressBook = addressBook;
		
		//create the sublist
		this.subList = new ArrayList<Contact>();
	}
	
	/**
	 *  @return            Returns the addressBook  
	 */
	public List<Contact> getAddressBook() {
		return addressBook;
	}
	
	/**
	 *  @return            Returns the subList  
	 */
	public List<Contact> getSubList() {
		return subList;
	}
	
	/**
	 * Method to set the subList.
	 * <P>
	 * 
	 * @param      index 	index of the contact selected in the list
	 */
	public void setSubList(int index) {
		subList.add(addressBook.get(index));
		
		// Notify the listener of the change in subList

	}
		
	/** Register an action event listener */
	public synchronized void addActionListener(ActionListener l) {
		if (actionListenerList == null) {
			actionListenerList = new ArrayList<ActionListener>();
		}
		actionListenerList.add(l);
	}
	
	/** Remove an action event listener */
	public synchronized void removeActionListener(ActionListener l) {
		if(actionListenerList != null && actionListenerList.contains(l)) {
			actionListenerList.remove(l);
		}
	}
	
	/** Fire Event */
	private void processEvent(ActionEvent e) {
		ArrayList<ActionListener> list;
		synchronized (this) {
			if(actionListenerList == null) return;
			//Do not worry about the cast warning here.
			list = (ArrayList<ActionListener>) actionListenerList.clone();
		}
		for (int i = 0; i < list.size(); i++) {
			ActionListener listener = list.get(i);
			listener.actionPerformed(e);
		}
	}
	
	/**
	 * This method reads in a specified comma delimited file. 
	 * The file is read one line at a time. The read line is then
	 * split and a Contact object is created using the parsed
	 * line.
	 * The Contact object is then added to a List, which is returned
	 * when the entire file has been processed.
	 * <P>
	 * 
	 * @param      filename     String 		name of the file to be read
	 * @return            		Returns a List of Contact objects  
	 */
	
	public void readFile(String filename) throws FileNotFoundException, IOException {
		//create a file object
		File file = new File(filename);
		
		//check if a specified file exists, if not then exit the program
		if(!file.exists()) {
			System.out.println("ERROR:: File not found.");
			System.exit(0);
		}
		
		//create file reader and buffered reader to read the file
		FileReader fileReader = new FileReader(filename);
		BufferedReader br = new BufferedReader(fileReader);
		
		List<Contact> addressBook = new ArrayList<Contact>();
		
		//read a line form the file, if the first is empty exit the program 
		String line = br.readLine();
		if(line.length() <= 0) {
			System.out.println("ERROR:: There is no data in file.");
			System.exit(0);
		}
		
		//read file one line at a time, until the end f file
		do {
			Contact contact = new Contact(line.split( "," ));
			addressBook.add(contact);
			line = br.readLine();
		} while(line != null);
		
		// Notify the listener of the change in addressBook
		this.addressBook = addressBook;
		processEvent(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "addressBook"));
	}	
}